@@ -1,5 +1,5 @@
 /*
- * $PostgreSQL: pgsql/contrib/pgstattuple/pgstattuple.c,v 1.17.4.1 2009/03/31 22:56:18 tgl Exp $
+ * $PostgreSQL: pgsql/contrib/pgstattuple/pgstattuple.c,v 1.20.2.1 2009/03/31 22:56:05 tgl Exp $
  *
  * Copyright (c) 2001,2002	Tatsuo Ishii
  *
@@ -38,7 +38,8 @@ PG_FUNCTION_INFO_V1(pgstattuplebyid);
 extern Datum pgstattuple(PG_FUNCTION_ARGS);
 extern Datum pgstattuplebyid(PG_FUNCTION_ARGS);
 
-static Datum pgstattuple_real(Relation rel);
+static Datum pgstattuple_real(Relation rel, FunctionCallInfo fcinfo);
+
 
 /* ----------
  * pgstattuple:
@@ -50,7 +51,6 @@ static Datum pgstattuple_real(Relation rel);
  * ----------
  */
 
-#define DUMMY_TUPLE "public.pgstattuple_type"
 #define NCOLUMNS 9
 #define NCHARS 32
 
@@ -63,11 +63,10 @@ pgstattuple(PG_FUNCTION_ARGS)
 	Datum		result;
 
 	/* open relation */
-	relrv = makeRangeVarFromNameList(textToQualifiedNameList(relname,
-														 "pgstattuple"));
+	relrv = makeRangeVarFromNameList(textToQualifiedNameList(relname));
 	rel = heap_openrv(relrv, AccessShareLock);
 
-	result = pgstattuple_real(rel);
+	result = pgstattuple_real(rel, fcinfo);
 
 	PG_RETURN_DATUM(result);
 }
@@ -82,7 +81,7 @@ pgstattuplebyid(PG_FUNCTION_ARGS)
 	/* open relation */
 	rel = heap_open(relid, AccessShareLock);
 
-	result = pgstattuple_real(rel);
+	result = pgstattuple_real(rel, fcinfo);
 
 	PG_RETURN_DATUM(result);
 }
@@ -93,7 +92,7 @@ pgstattuplebyid(PG_FUNCTION_ARGS)
  * The real work occurs here
  */
 static Datum
-pgstattuple_real(Relation rel)
+pgstattuple_real(Relation rel, FunctionCallInfo fcinfo)
 {
 	HeapScanDesc scan;
 	HeapTuple	tuple;
@@ -126,14 +125,16 @@ pgstattuple_real(Relation rel)
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				 errmsg("cannot access temporary tables of other sessions")));
 
-	/*
-	 * Build a tuple description for a pgstattupe_type tuple
-	 */
-	tupdesc = RelationNameGetTupleDesc(DUMMY_TUPLE);
+	/* Build a tuple descriptor for our result type */
+	if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE)
+		elog(ERROR, "return type must be a row type");
+
+	/* make sure we have a persistent copy of the tupdesc */
+	tupdesc = CreateTupleDescCopy(tupdesc);
 
 	/*
-	 * Generate attribute metadata needed later to produce tuples from raw
-	 * C strings
+	 * Generate attribute metadata needed later to produce tuples from raw C
+	 * strings
 	 */
 	attinmeta = TupleDescGetAttInMetadata(tupdesc);
 
@@ -206,9 +207,9 @@ pgstattuple_real(Relation rel)
 	}
 
 	/*
-	 * Prepare a values array for constructing the tuple. This should be
-	 * an array of C strings which will be processed later by the
-	 * appropriate "in" functions.
+	 * Prepare a values array for constructing the tuple. This should be an
+	 * array of C strings which will be processed later by the appropriate
+	 * "in" functions.
 	 */
 	values = (char **) palloc(NCOLUMNS * sizeof(char *));
 	for (i = 0; i < NCOLUMNS; i++)
