@@ -7,8 +7,6 @@
 #include "funcapi.h"
 #include "miscadmin.h"
 
-#ifdef USE_LIBXSLT
-
 /* libxml includes */
 
 #include <libxml/xpath.h>
@@ -22,35 +20,30 @@
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 
-#endif /* USE_LIBXSLT */
-
-
-/* externally accessible functions */
-
-Datum		xslt_process(PG_FUNCTION_ARGS);
-
-#ifdef USE_LIBXSLT
 
 /* declarations to come from xpath.c */
-extern void elog_error(const char *explain, bool force);
-extern void pgxml_parser_init(void);
+
+extern void elog_error(int level, char *explain, int force);
+extern void pgxml_parser_init();
+extern xmlChar *pgxml_texttoxmlchar(text *textstring);
+
+#define GET_STR(textp) DatumGetCString(DirectFunctionCall1(textout, PointerGetDatum(textp)))
 
 /* local defs */
 static void parse_params(const char **params, text *paramstr);
 
+Datum		xslt_process(PG_FUNCTION_ARGS);
+
+
 #define MAXPARAMS 20			/* must be even, see parse_params() */
 
-#define GET_STR(textp) DatumGetCString(DirectFunctionCall1(textout, PointerGetDatum(textp)))
-
-#endif /* USE_LIBXSLT */
-
 
 PG_FUNCTION_INFO_V1(xslt_process);
 
 Datum
 xslt_process(PG_FUNCTION_ARGS)
 {
-#ifdef USE_LIBXSLT
+
 
 	const char *params[MAXPARAMS + 1];	/* +1 for the terminator */
 	xsltStylesheetPtr stylesheet = NULL;
@@ -66,13 +59,14 @@ xslt_process(PG_FUNCTION_ARGS)
 	text	   *paramstr;
 	text	   *tres;
 
+
 	if (fcinfo->nargs == 3)
 	{
 		paramstr = PG_GETARG_TEXT_P(2);
 		parse_params(params, paramstr);
 	}
 	else
-	/* No parameters */
+		/* No parameters */
 		params[0] = NULL;
 
 	/* Setup parser */
@@ -87,7 +81,8 @@ xslt_process(PG_FUNCTION_ARGS)
 
 	if (doctree == NULL)
 	{
-		elog_error("Error parsing XML document", false);
+		xmlCleanupParser();
+		elog_error(ERROR, "Error parsing XML document", 0);
 
 		PG_RETURN_NULL();
 	}
@@ -100,7 +95,8 @@ xslt_process(PG_FUNCTION_ARGS)
 		if (ssdoc == NULL)
 		{
 			xmlFreeDoc(doctree);
-			elog_error("Error parsing stylesheet as XML document", false);
+			xmlCleanupParser();
+			elog_error(ERROR, "Error parsing stylesheet as XML document", 0);
 			PG_RETURN_NULL();
 		}
 
@@ -114,7 +110,8 @@ xslt_process(PG_FUNCTION_ARGS)
 	{
 		xmlFreeDoc(doctree);
 		xsltCleanupGlobals();
-		elog_error("Failed to parse stylesheet", false);
+		xmlCleanupParser();
+		elog_error(ERROR, "Failed to parse stylesheet", 0);
 		PG_RETURN_NULL();
 	}
 
@@ -126,6 +123,7 @@ xslt_process(PG_FUNCTION_ARGS)
 	xmlFreeDoc(doctree);
 
 	xsltCleanupGlobals();
+	xmlCleanupParser();
 
 	if (resstat < 0)
 		PG_RETURN_NULL();
@@ -135,18 +133,8 @@ xslt_process(PG_FUNCTION_ARGS)
 	VARATT_SIZEP(tres) = reslen + VARHDRSZ;
 
 	PG_RETURN_TEXT_P(tres);
-
-#else /* !USE_LIBXSLT */
-
-	ereport(ERROR,
-			(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-			 errmsg("xslt_process() is not available without libxslt")));
-	PG_RETURN_NULL();
-
-#endif /* USE_LIBXSLT */
 }
 
-#ifdef USE_LIBXSLT
 
 static void
 parse_params(const char **params, text *paramstr)
@@ -195,5 +183,3 @@ parse_params(const char **params, text *paramstr)
 
 	params[i] = NULL;
 }
-
-#endif /* USE_LIBXSLT */
