@@ -2,160 +2,31 @@
  *
  * xlogutils.c
  *
+ * PostgreSQL transaction log manager utility routines
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * This file contains support routines that are used by XLOG replay functions.
+ * None of this code is used during normal system operation.
+ *
+ *
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Header: /cvsroot/pgsql/src/backend/access/transam/xlogutils.c,v 1.26 2003/08/04 02:39:57 momjian Exp $
+ * $PostgreSQL: pgsql/src/backend/access/transam/xlogutils.c,v 1.39 2005/10/15 02:49:11 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
-#include "access/htup.h"
 #include "access/xlogutils.h"
-#include "catalog/pg_database.h"
-#include "storage/bufpage.h"
+#include "storage/bufmgr.h"
 #include "storage/smgr.h"
 #include "utils/hsearch.h"
-#include "utils/relcache.h"
 
 
 /*
- * ---------------------------------------------------------------
- *
- * Index support functions
- *
- *----------------------------------------------------------------
- */
-
-/*
- * Check if specified heap tuple was inserted by given
- * xaction/command and return
- *
- * - -1 if not
- * - 0	if there is no tuple at all
- * - 1	if yes
- */
-int
-XLogIsOwnerOfTuple(RelFileNode hnode, ItemPointer iptr,
-				   TransactionId xid, CommandId cid)
-{
-	Relation	reln;
-	Buffer		buffer;
-	Page		page;
-	ItemId		lp;
-	HeapTupleHeader htup;
-
-	reln = XLogOpenRelation(false, RM_HEAP_ID, hnode);
-	if (!RelationIsValid(reln))
-		return (0);
-
-	buffer = ReadBuffer(reln, ItemPointerGetBlockNumber(iptr));
-	if (!BufferIsValid(buffer))
-		return (0);
-
-	LockBuffer(buffer, BUFFER_LOCK_SHARE);
-	page = (Page) BufferGetPage(buffer);
-	if (PageIsNew((PageHeader) page) ||
-		ItemPointerGetOffsetNumber(iptr) > PageGetMaxOffsetNumber(page))
-	{
-		UnlockAndReleaseBuffer(buffer);
-		return (0);
-	}
-	lp = PageGetItemId(page, ItemPointerGetOffsetNumber(iptr));
-	if (!ItemIdIsUsed(lp) || ItemIdDeleted(lp))
-	{
-		UnlockAndReleaseBuffer(buffer);
-		return (0);
-	}
-
-	htup = (HeapTupleHeader) PageGetItem(page, lp);
-
-	Assert(PageGetSUI(page) == ThisStartUpID);
-	if (!TransactionIdEquals(HeapTupleHeaderGetXmin(htup), xid) ||
-		HeapTupleHeaderGetCmin(htup) != cid)
-	{
-		UnlockAndReleaseBuffer(buffer);
-		return (-1);
-	}
-
-	UnlockAndReleaseBuffer(buffer);
-	return (1);
-}
-
-/*
- * MUST BE CALLED ONLY ON RECOVERY.
- *
- * Check if exists valid (inserted by not aborted xaction) heap tuple
- * for given item pointer
- */
-bool
-XLogIsValidTuple(RelFileNode hnode, ItemPointer iptr)
-{
-	Relation	reln;
-	Buffer		buffer;
-	Page		page;
-	ItemId		lp;
-	HeapTupleHeader htup;
-
-	reln = XLogOpenRelation(false, RM_HEAP_ID, hnode);
-	if (!RelationIsValid(reln))
-		return (false);
-
-	buffer = ReadBuffer(reln, ItemPointerGetBlockNumber(iptr));
-	if (!BufferIsValid(buffer))
-		return (false);
-
-	LockBuffer(buffer, BUFFER_LOCK_SHARE);
-	page = (Page) BufferGetPage(buffer);
-	if (PageIsNew((PageHeader) page) ||
-		ItemPointerGetOffsetNumber(iptr) > PageGetMaxOffsetNumber(page))
-	{
-		UnlockAndReleaseBuffer(buffer);
-		return (false);
-	}
-
-	if (PageGetSUI(page) != ThisStartUpID)
-	{
-		Assert(PageGetSUI(page) < ThisStartUpID);
-		UnlockAndReleaseBuffer(buffer);
-		return (true);
-	}
-
-	lp = PageGetItemId(page, ItemPointerGetOffsetNumber(iptr));
-	if (!ItemIdIsUsed(lp) || ItemIdDeleted(lp))
-	{
-		UnlockAndReleaseBuffer(buffer);
-		return (false);
-	}
-
-	htup = (HeapTupleHeader) PageGetItem(page, lp);
-
-	/* MUST CHECK WASN'T TUPLE INSERTED IN PREV STARTUP */
-
-	if (!(htup->t_infomask & HEAP_XMIN_COMMITTED))
-	{
-		if (htup->t_infomask & HEAP_XMIN_INVALID ||
-			(htup->t_infomask & HEAP_MOVED_IN &&
-			 TransactionIdDidAbort(HeapTupleHeaderGetXvac(htup))) ||
-			TransactionIdDidAbort(HeapTupleHeaderGetXmin(htup)))
-		{
-			UnlockAndReleaseBuffer(buffer);
-			return (false);
-		}
-	}
-
-	UnlockAndReleaseBuffer(buffer);
-	return (true);
-}
-
-/*
- * ---------------------------------------------------------------
  *
  * Storage related support functions
  *
- *----------------------------------------------------------------
  */
 
 Buffer
@@ -189,8 +60,10 @@ XLogReadBuffer(bool extend, Relation reln, BlockNumber blkno)
 	return (buffer);
 }
 
+
 /*
- * "Relation" cache
+ * Lightweight "Relation" cache --- this substitutes for the normal relcache
+ * during XLOG replay.
  */
 
 typedef struct XLogRelDesc
@@ -248,18 +121,15 @@ _xl_remove_hash_entry(XLogRelDesc *rdesc)
 	rdesc->moreRecently->lessRecently = rdesc->lessRecently;
 
 	hentry = (XLogRelCacheEntry *) hash_search(_xlrelcache,
-				  (void *) &(rdesc->reldata.rd_node), HASH_REMOVE, NULL);
+					  (void *) &(rdesc->reldata.rd_node), HASH_REMOVE, NULL);
 	if (hentry == NULL)
 		elog(PANIC, "_xl_remove_hash_entry: file was not found in cache");
 
-	if (rdesc->reldata.rd_fd >= 0)
-		smgrclose(DEFAULT_SMGR, &(rdesc->reldata));
+	RelationCloseSmgr(&(rdesc->reldata));
 
 	memset(rdesc, 0, sizeof(XLogRelDesc));
 	memset(tpgc, 0, sizeof(FormData_pg_class));
 	rdesc->reldata.rd_rel = tpgc;
-
-	return;
 }
 
 static XLogRelDesc *
@@ -287,7 +157,6 @@ _xl_new_reldesc(void)
 void
 XLogInitRelationCache(void)
 {
-	CreateDummyCaches();
 	_xl_init_rel_cache();
 }
 
@@ -297,8 +166,6 @@ XLogCloseRelationCache(void)
 	HASH_SEQ_STATUS status;
 	XLogRelCacheEntry *hentry;
 
-	DestroyDummyCaches();
-
 	if (!_xlrelarr)
 		return;
 
@@ -315,8 +182,11 @@ XLogCloseRelationCache(void)
 	_xlrelarr = NULL;
 }
 
+/*
+ * Open a relation during XLOG replay
+ */
 Relation
-XLogOpenRelation(bool redo, RmgrId rmid, RelFileNode rnode)
+XLogOpenRelation(RelFileNode rnode)
 {
 	XLogRelDesc *res;
 	XLogRelCacheEntry *hentry;
@@ -338,26 +208,40 @@ XLogOpenRelation(bool redo, RmgrId rmid, RelFileNode rnode)
 
 		sprintf(RelationGetRelationName(&(res->reldata)), "%u", rnode.relNode);
 
-		/* unexisting DB id */
-		res->reldata.rd_lockInfo.lockRelId.dbId = RecoveryDb;
-		res->reldata.rd_lockInfo.lockRelId.relId = rnode.relNode;
 		res->reldata.rd_node = rnode;
 
+		/*
+		 * We set up the lockRelId in case anything tries to lock the dummy
+		 * relation.  Note that this is fairly bogus since relNode may be
+		 * different from the relation's OID.  It shouldn't really matter
+		 * though, since we are presumably running by ourselves and can't have
+		 * any lock conflicts ...
+		 */
+		res->reldata.rd_lockInfo.lockRelId.dbId = rnode.dbNode;
+		res->reldata.rd_lockInfo.lockRelId.relId = rnode.relNode;
+
 		hentry = (XLogRelCacheEntry *)
 			hash_search(_xlrelcache, (void *) &rnode, HASH_ENTER, &found);
 
-		if (hentry == NULL)
-			elog(PANIC, "XLogOpenRelation: out of memory for cache");
-
 		if (found)
 			elog(PANIC, "XLogOpenRelation: file found on insert into cache");
 
 		hentry->rdesc = res;
 
 		res->reldata.rd_targblock = InvalidBlockNumber;
-		res->reldata.rd_fd = -1;
-		res->reldata.rd_fd = smgropen(DEFAULT_SMGR, &(res->reldata),
-									  true /* allow failure */ );
+		res->reldata.rd_smgr = NULL;
+		RelationOpenSmgr(&(res->reldata));
+
+		/*
+		 * Create the target file if it doesn't already exist.  This lets us
+		 * cope if the replay sequence contains writes to a relation that is
+		 * later deleted.  (The original coding of this routine would instead
+		 * return NULL, causing the writes to be suppressed. But that seems
+		 * like it risks losing valuable data if the filesystem loses an inode
+		 * during a crash.	Better to write the data until we are actually
+		 * told to delete the file.)
+		 */
+		smgrcreate(res->reldata.rd_smgr, res->reldata.rd_istemp, true);
 	}
 
 	res->moreRecently = &(_xlrelarr[0]);
@@ -365,8 +249,31 @@ XLogOpenRelation(bool redo, RmgrId rmid, RelFileNode rnode)
 	_xlrelarr[0].lessRecently = res;
 	res->lessRecently->moreRecently = res;
 
-	if (res->reldata.rd_fd < 0) /* file doesn't exist */
-		return (NULL);
-
 	return (&(res->reldata));
 }
+
+/*
+ * Close a relation during XLOG replay
+ *
+ * This is called when the relation is about to be deleted; we need to ensure
+ * that there is no dangling smgr reference in the xlog relation cache.
+ *
+ * Currently, we don't bother to physically remove the relation from the
+ * cache, we just let it age out normally.
+ */
+void
+XLogCloseRelation(RelFileNode rnode)
+{
+	XLogRelDesc *rdesc;
+	XLogRelCacheEntry *hentry;
+
+	hentry = (XLogRelCacheEntry *)
+		hash_search(_xlrelcache, (void *) &rnode, HASH_FIND, NULL);
+
+	if (!hentry)
+		return;					/* not in cache so no work */
+
+	rdesc = hentry->rdesc;
+
+	RelationCloseSmgr(&(rdesc->reldata));
+}
