@@ -6,7 +6,7 @@
  * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- *	$PostgreSQL: pgsql/src/backend/executor/execAmi.c,v 1.85 2005/10/15 02:49:16 momjian Exp $
+ *	$PostgreSQL: pgsql/src/backend/executor/execAmi.c,v 1.85.2.3 2008/08/05 21:28:48 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -65,7 +65,19 @@ ExecReScan(PlanState *node, ExprContext *exprCtxt)
 	if (node->instrument)
 		InstrEndLoop(node->instrument);
 
-	/* If we have changed parameters, propagate that info */
+	/*
+	 * If we have changed parameters, propagate that info.
+	 *
+	 * Note: ExecReScanSetParamPlan() can add bits to node->chgParam,
+	 * corresponding to the output param(s) that the InitPlan will update.
+	 * Since we make only one pass over the list, that means that an InitPlan
+	 * can depend on the output param(s) of a sibling InitPlan only if that
+	 * sibling appears earlier in the list.  This is workable for now given
+	 * the limited ways in which one InitPlan could depend on another, but
+	 * eventually we might need to work harder (or else make the planner
+	 * enlarge the extParam/allParam sets to include the params of depended-on
+	 * InitPlans).
+	 */
 	if (node->chgParam != NULL)
 	{
 		ListCell   *l;
@@ -371,9 +383,6 @@ ExecSupportsBackwardScan(Plan *node)
 		case T_Sort:
 			return true;
 
-		case T_Unique:
-			return ExecSupportsBackwardScan(outerPlan(node));
-
 		case T_Limit:
 			return ExecSupportsBackwardScan(outerPlan(node));
 
@@ -403,9 +412,9 @@ ExecMayReturnRawTuples(PlanState *node)
 	 * but just pass up input tuples, we have to recursively examine the input
 	 * plan node.
 	 *
-	 * Note: Hash and Material are listed here because they sometimes return an
-	 * original input tuple, not a copy.  But Sort and SetOp never return an
-	 * original tuple, so they can be treated like projecting nodes.
+	 * Note: Hash and Material are listed here because they sometimes return
+	 * an original input tuple, not a copy.  But Sort and SetOp never return
+	 * an original tuple, so they can be treated like projecting nodes.
 	 */
 	switch (nodeTag(node))
 	{
