@@ -1,15 +1,15 @@
 /*-------------------------------------------------------------------------
  *
  * nodes.c
- *	  support code for nodes (now that we get rid of the home-brew
- *	  inheritance system, our support code for nodes get much simpler)
+ *	  support code for nodes (now that we have removed the home-brew
+ *	  inheritance system, our support code for nodes is much simpler)
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/nodes/nodes.c,v 1.21 2003/08/04 02:39:59 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/nodes/nodes.c,v 1.25 2004/12/31 21:59:55 pgsql Exp $
  *
  * HISTORY
  *	  Andrew Yu			Oct 20, 1994	file creation
