@@ -3,12 +3,12 @@
  * tlist.c
  *	  Target list manipulation routines
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/optimizer/util/tlist.c,v 1.51 2001/10/25 05:49:34 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/optimizer/util/tlist.c,v 1.70 2005/10/15 02:49:21 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,21 +17,22 @@
 #include "nodes/makefuncs.h"
 #include "optimizer/tlist.h"
 #include "optimizer/var.h"
+#include "parser/parse_expr.h"
 
 
 /*****************************************************************************
- *	---------- RELATION node target list routines ----------
+ *		Target list creation and searching utilities
  *****************************************************************************/
 
 /*
- * tlistentry_member
+ * tlist_member
  *	  Finds the (first) member of the given tlist whose expression is
  *	  equal() to the given expression.	Result is NULL if no such member.
  */
 TargetEntry *
-tlistentry_member(Node *node, List *targetlist)
+tlist_member(Node *node, List *targetlist)
 {
-	List	   *temp;
+	ListCell   *temp;
 
 	foreach(temp, targetlist)
 	{
@@ -43,106 +44,6 @@ tlistentry_member(Node *node, List *targetlist)
 	return NULL;
 }
 
-#ifdef NOT_USED
-/*
- * matching_tlist_expr
- *	  Same as tlistentry_member(), except returns the tlist expression
- *	  rather than its parent TargetEntry node.
- */
-Node *
-matching_tlist_expr(Node *node, List *targetlist)
-{
-	TargetEntry *tlentry;
-
-	tlentry = tlistentry_member(node, targetlist);
-	if (tlentry)
-		return tlentry->expr;
-
-	return (Node *) NULL;
-}
-#endif
-
-/*
- * tlist_member
- *	  Same as tlistentry_member(), except returns the Resdom node
- *	  rather than its parent TargetEntry node.
- */
-Resdom *
-tlist_member(Node *node, List *targetlist)
-{
-	TargetEntry *tlentry;
-
-	tlentry = tlistentry_member(node, targetlist);
-	if (tlentry)
-		return tlentry->resdom;
-
-	return (Resdom *) NULL;
-}
-
-/*
- * add_var_to_tlist
- *	  Creates a targetlist entry corresponding to the supplied var node
- *	  'var' and adds the new targetlist entry to the targetlist field of
- *	  'rel'.  No entry is created if 'var' is already in the tlist.
- */
-void
-add_var_to_tlist(RelOptInfo *rel, Var *var)
-{
-	if (!tlistentry_member((Node *) var, rel->targetlist))
-	{
-		/* XXX is copyObject necessary here? */
-		rel->targetlist = lappend(rel->targetlist,
-							   create_tl_element((Var *) copyObject(var),
-										   length(rel->targetlist) + 1));
-	}
-}
-
-/*
- * create_tl_element
- *	  Creates a target list entry node and its associated (resdom var) pair
- *	  with its resdom number equal to 'resdomno'.
- */
-TargetEntry *
-create_tl_element(Var *var, int resdomno)
-{
-	return makeTargetEntry(makeResdom(resdomno,
-									  var->vartype,
-									  var->vartypmod,
-									  NULL,
-									  false),
-						   (Node *) var);
-}
-
-/*****************************************************************************
- *		---------- GENERAL target list routines ----------
- *****************************************************************************/
-
-/*
- * new_unsorted_tlist
- *	  Creates a copy of a target list by creating new resdom nodes
- *	  without sort information.
- *
- * 'targetlist' is the target list to be copied.
- *
- * Returns the resulting target list.
- *
- */
-List *
-new_unsorted_tlist(List *targetlist)
-{
-	List	   *new_targetlist = (List *) copyObject((Node *) targetlist);
-	List	   *x;
-
-	foreach(x, new_targetlist)
-	{
-		TargetEntry *tle = (TargetEntry *) lfirst(x);
-
-		tle->resdom->reskey = 0;
-		tle->resdom->reskeyop = (Oid) 0;
-	}
-	return new_targetlist;
-}
-
 /*
  * flatten_tlist
  *	  Create a target list that only contains unique variables.
@@ -165,7 +66,7 @@ flatten_tlist(List *tlist)
 	List	   *new_tlist;
 
 	new_tlist = add_to_flat_tlist(NIL, vlist);
-	freeList(vlist);
+	list_free(vlist);
 	return new_tlist;
 }
 
@@ -181,38 +82,27 @@ flatten_tlist(List *tlist)
 List *
 add_to_flat_tlist(List *tlist, List *vars)
 {
-	int			next_resdomno = length(tlist) + 1;
-	List	   *v;
+	int			next_resno = list_length(tlist) + 1;
+	ListCell   *v;
 
 	foreach(v, vars)
 	{
-		Var		   *var = lfirst(v);
+		Var		   *var = (Var *) lfirst(v);
 
-		if (!tlistentry_member((Node *) var, tlist))
+		if (!tlist_member((Node *) var, tlist))
 		{
-			Resdom	   *r;
+			TargetEntry *tle;
 
-			r = makeResdom(next_resdomno++,
-						   var->vartype,
-						   var->vartypmod,
-						   NULL,
-						   false);
-			tlist = lappend(tlist,
-							makeTargetEntry(r, copyObject(var)));
+			tle = makeTargetEntry(copyObject(var),		/* copy needed?? */
+								  next_resno++,
+								  NULL,
+								  false);
+			tlist = lappend(tlist, tle);
 		}
 	}
 	return tlist;
 }
 
-Var *
-get_expr(TargetEntry *tle)
-{
-	Assert(tle != NULL);
-	Assert(tle->expr != NULL);
-
-	return (Var *) tle->expr;
-}
-
 /*
  * get_sortgroupclause_tle
  *		Find the targetlist entry matching the given SortClause
@@ -226,17 +116,17 @@ get_sortgroupclause_tle(SortClause *sortClause,
 						List *targetList)
 {
 	Index		refnumber = sortClause->tleSortGroupRef;
-	List	   *l;
+	ListCell   *l;
 
 	foreach(l, targetList)
 	{
 		TargetEntry *tle = (TargetEntry *) lfirst(l);
 
-		if (tle->resdom->ressortgroupref == refnumber)
+		if (tle->ressortgroupref == refnumber)
 			return tle;
 	}
 
-	elog(ERROR, "get_sortgroupclause_tle: ORDER/GROUP BY expression not found in targetlist");
+	elog(ERROR, "ORDER/GROUP BY expression not found in targetlist");
 	return NULL;				/* keep compiler quiet */
 }
 
@@ -253,5 +143,27 @@ get_sortgroupclause_expr(SortClause *sortClause, List *targetList)
 {
 	TargetEntry *tle = get_sortgroupclause_tle(sortClause, targetList);
 
-	return tle->expr;
+	return (Node *) tle->expr;
+}
+
+/*
+ * get_sortgrouplist_exprs
+ *		Given a list of SortClauses (or GroupClauses), build a list
+ *		of the referenced targetlist expressions.
+ */
+List *
+get_sortgrouplist_exprs(List *sortClauses, List *targetList)
+{
+	List	   *result = NIL;
+	ListCell   *l;
+
+	foreach(l, sortClauses)
+	{
+		SortClause *sortcl = (SortClause *) lfirst(l);
+		Node	   *sortexpr;
+
+		sortexpr = get_sortgroupclause_expr(sortcl, targetList);
+		result = lappend(result, sortexpr);
+	}
+	return result;
 }
