@@ -13,7 +13,7 @@
  *
  *	Copyright (c) 2001-2005, PostgreSQL Global Development Group
  *
- *	$PostgreSQL: pgsql/src/backend/postmaster/pgstat.c,v 1.111.2.3 2006/05/19 15:15:38 alvherre Exp $
+ *	$PostgreSQL: pgsql/src/backend/postmaster/pgstat.c,v 1.111.2.9 2007/07/08 22:23:32 tgl Exp $
  * ----------
  */
 #include "postgres.h"
@@ -178,6 +178,7 @@ static void pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 					  PgStat_StatBeEntry **betab,
 					  int *numbackends);
 static void backend_read_statsfile(void);
+static HTAB *pgstat_collect_oids(Oid catalogid);
 
 static void pgstat_setheader(PgStat_MsgHdr *hdr, StatMsgType mtype);
 static void pgstat_send(void *msg, int len);
@@ -339,8 +340,12 @@ pgstat_init(void)
 		 * rules prevent it).
 		 */
 		test_byte = TESTBYTEVAL;
+
+retry1:
 		if (send(pgStatSock, &test_byte, 1, 0) != 1)
 		{
+			if (errno == EINTR)
+				goto retry1;	/* if interrupted, just retry */
 			ereport(LOG,
 					(errcode_for_socket_access(),
 					 errmsg("could not send test message on socket for statistics collector: %m")));
@@ -391,8 +396,11 @@ pgstat_init(void)
 
 		test_byte++;			/* just make sure variable is changed */
 
+retry2:
 		if (recv(pgStatSock, &test_byte, 1, 0) != 1)
 		{
+			if (errno == EINTR)
+				goto retry2;	/* if interrupted, just retry */
 			ereport(LOG,
 					(errcode_for_socket_access(),
 					 errmsg("could not receive test message on socket for statistics collector: %m")));
@@ -879,10 +887,7 @@ pgstat_report_tabstat(void)
 void
 pgstat_vacuum_tabstat(void)
 {
-	List	   *oidlist;
-	Relation	rel;
-	HeapScanDesc scan;
-	HeapTuple	tup;
+	HTAB	   *htab;
 	PgStat_MsgTabpurge msg;
 	HASH_SEQ_STATUS hstat;
 	PgStat_StatDBEntry *dbentry;
@@ -901,15 +906,7 @@ pgstat_vacuum_tabstat(void)
 	/*
 	 * Read pg_database and make a list of OIDs of all existing databases
 	 */
-	oidlist = NIL;
-	rel = heap_open(DatabaseRelationId, AccessShareLock);
-	scan = heap_beginscan(rel, SnapshotNow, 0, NULL);
-	while ((tup = heap_getnext(scan, ForwardScanDirection)) != NULL)
-	{
-		oidlist = lappend_oid(oidlist, HeapTupleGetOid(tup));
-	}
-	heap_endscan(scan);
-	heap_close(rel, AccessShareLock);
+	htab = pgstat_collect_oids(DatabaseRelationId);
 
 	/*
 	 * Search the database hash table for dead databases and tell the
@@ -920,12 +917,16 @@ pgstat_vacuum_tabstat(void)
 	{
 		Oid			dbid = dbentry->databaseid;
 
-		if (!list_member_oid(oidlist, dbid))
+		CHECK_FOR_INTERRUPTS();
+
+		/* the DB entry for shared tables (with InvalidOid) is never dropped */
+		if (OidIsValid(dbid) &&
+			hash_search(htab, (void *) &dbid, HASH_FIND, NULL) == NULL)
 			pgstat_drop_database(dbid);
 	}
 
 	/* Clean up */
-	list_free(oidlist);
+	hash_destroy(htab);
 
 	/*
 	 * Lookup our own database entry; if not found, nothing more to do.
@@ -939,15 +940,7 @@ pgstat_vacuum_tabstat(void)
 	/*
 	 * Similarly to above, make a list of all known relations in this DB.
 	 */
-	oidlist = NIL;
-	rel = heap_open(RelationRelationId, AccessShareLock);
-	scan = heap_beginscan(rel, SnapshotNow, 0, NULL);
-	while ((tup = heap_getnext(scan, ForwardScanDirection)) != NULL)
-	{
-		oidlist = lappend_oid(oidlist, HeapTupleGetOid(tup));
-	}
-	heap_endscan(scan);
-	heap_close(rel, AccessShareLock);
+	htab = pgstat_collect_oids(RelationRelationId);
 
 	/*
 	 * Initialize our messages table counter to zero
@@ -960,13 +953,17 @@ pgstat_vacuum_tabstat(void)
 	hash_seq_init(&hstat, dbentry->tables);
 	while ((tabentry = (PgStat_StatTabEntry *) hash_seq_search(&hstat)) != NULL)
 	{
-		if (list_member_oid(oidlist, tabentry->tableid))
+		Oid			tabid = tabentry->tableid;
+
+		CHECK_FOR_INTERRUPTS();
+
+		if (hash_search(htab, (void *) &tabid, HASH_FIND, NULL) != NULL)
 			continue;
 
 		/*
 		 * Not there, so add this table's Oid to the message
 		 */
-		msg.m_tableid[msg.m_nentries++] = tabentry->tableid;
+		msg.m_tableid[msg.m_nentries++] = tabid;
 
 		/*
 		 * If the message is full, send it out and reinitialize to empty
@@ -998,7 +995,50 @@ pgstat_vacuum_tabstat(void)
 	}
 
 	/* Clean up */
-	list_free(oidlist);
+	hash_destroy(htab);
+}
+
+
+/* ----------
+ * pgstat_collect_oids() -
+ *
+ *	Collect the OIDs of either all databases or all tables, according to
+ *	the parameter, into a temporary hash table.  Caller should hash_destroy
+ *	the result when done with it.
+ * ----------
+ */
+static HTAB *
+pgstat_collect_oids(Oid catalogid)
+{
+	HTAB	   *htab;
+	HASHCTL		hash_ctl;
+	Relation	rel;
+	HeapScanDesc scan;
+	HeapTuple	tup;
+
+	memset(&hash_ctl, 0, sizeof(hash_ctl));
+	hash_ctl.keysize = sizeof(Oid);
+	hash_ctl.entrysize = sizeof(Oid);
+	hash_ctl.hash = oid_hash;
+	htab = hash_create("Temporary table of OIDs",
+					   PGSTAT_TAB_HASH_SIZE,
+					   &hash_ctl,
+					   HASH_ELEM | HASH_FUNCTION);
+
+	rel = heap_open(catalogid, AccessShareLock);
+	scan = heap_beginscan(rel, SnapshotNow, 0, NULL);
+	while ((tup = heap_getnext(scan, ForwardScanDirection)) != NULL)
+	{
+		Oid		thisoid = HeapTupleGetOid(tup);
+
+		CHECK_FOR_INTERRUPTS();
+
+		(void) hash_search(htab, (void *) &thisoid, HASH_ENTER, NULL);
+	}
+	heap_endscan(scan);
+	heap_close(rel, AccessShareLock);
+
+	return htab;
 }
 
 
@@ -1030,8 +1070,12 @@ pgstat_drop_database(Oid databaseid)
  *	Tell the collector that we just dropped a relation.
  *	(If the message gets lost, we will still clean the dead entry eventually
  *	via future invocations of pgstat_vacuum_tabstat().)
+ *
+ *	Currently not used for lack of any good place to call it; we rely
+ *	entirely on pgstat_vacuum_tabstat() to clean out stats for dead rels.
  * ----------
  */
+#ifdef NOT_USED
 void
 pgstat_drop_relation(Oid relid)
 {
@@ -1050,6 +1094,7 @@ pgstat_drop_relation(Oid relid)
 	msg.m_databaseid = MyDatabaseId;
 	pgstat_send(&msg, len);
 }
+#endif /* NOT_USED */
 
 
 /* ----------
@@ -1431,17 +1476,23 @@ pgstat_setheader(PgStat_MsgHdr *hdr, StatMsgType mtype)
 static void
 pgstat_send(void *msg, int len)
 {
+	int			rc;
+
 	if (pgStatSock < 0)
 		return;
 
 	((PgStat_MsgHdr *) msg)->m_size = len;
 
+	/* We'll retry after EINTR, but ignore all other failures */
+	do
+	{
+		rc = send(pgStatSock, msg, len, 0);
+	} while (rc < 0 && errno == EINTR);
+
 #ifdef USE_ASSERT_CHECKING
-	if (send(pgStatSock, msg, len, 0) < 0)
+	/* In debug builds, log send failures ... */
+	if (rc < 0)
 		elog(LOG, "could not send to statistics collector: %m");
-#else
-	send(pgStatSock, msg, len, 0);
-	/* We deliberately ignore any error from send() */
 #endif
 }
 
@@ -1972,9 +2023,13 @@ pgstat_recvbuffer(void)
 			len = recv(pgStatSock, (char *) &input_buffer,
 					   sizeof(PgStat_Msg), 0);
 			if (len < 0)
+			{
+				if (errno == EINTR)
+					continue;
 				ereport(ERROR,
 						(errcode_for_socket_access(),
 						 errmsg("could not read statistics message: %m")));
+			}
 
 			/*
 			 * We ignore messages that are smaller than our common header
@@ -2919,6 +2974,12 @@ pgstat_recv_vacuum(PgStat_MsgVacuum *msg, int len)
 	tabentry->n_dead_tuples = 0;
 	if (msg->m_analyze)
 		tabentry->last_anl_tuples = msg->m_tuples;
+	else
+	{
+		/* last_anl_tuples must never exceed n_live_tuples */
+		tabentry->last_anl_tuples = Min(tabentry->last_anl_tuples,
+										msg->m_tuples);
+	}
 }
 
 /* ----------
@@ -3055,7 +3116,8 @@ pgstat_recv_tabstat(PgStat_MsgTabstat *msg, int len)
 			tabentry->tuples_updated += tabmsg[i].t_tuples_updated;
 			tabentry->tuples_deleted += tabmsg[i].t_tuples_deleted;
 
-			tabentry->n_live_tuples += tabmsg[i].t_tuples_inserted;
+			tabentry->n_live_tuples += tabmsg[i].t_tuples_inserted -
+				tabmsg[i].t_tuples_deleted;
 			tabentry->n_dead_tuples += tabmsg[i].t_tuples_updated +
 				tabmsg[i].t_tuples_deleted;
 
