@@ -9,7 +9,7 @@
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/adt/varbit.c,v 1.44.4.1 2007/08/21 02:40:26 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/adt/varbit.c,v 1.47.2.1 2007/08/21 02:40:18 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -83,8 +83,8 @@ bit_in(PG_FUNCTION_ARGS)
 	else
 	{
 		/*
-		 * Otherwise it's binary.  This allows things like cast('1001' as
-		 * bit) to work transparently.
+		 * Otherwise it's binary.  This allows things like cast('1001' as bit)
+		 * to work transparently.
 		 */
 		bit_not_hex = true;
 		sp = input_string;
@@ -98,16 +98,16 @@ bit_in(PG_FUNCTION_ARGS)
 		bitlen = slen * 4;
 
 	/*
-	 * Sometimes atttypmod is not supplied. If it is supplied we need to
-	 * make sure that the bitstring fits.
+	 * Sometimes atttypmod is not supplied. If it is supplied we need to make
+	 * sure that the bitstring fits.
 	 */
 	if (atttypmod <= 0)
 		atttypmod = bitlen;
 	else if (bitlen != atttypmod)
 		ereport(ERROR,
 				(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
-			   errmsg("bit string length %d does not match type bit(%d)",
-					  bitlen, atttypmod)));
+				 errmsg("bit string length %d does not match type bit(%d)",
+						bitlen, atttypmod)));
 
 	len = VARBITTOTALLEN(atttypmod);
 	/* set to 0 so that *r is always initialised and string is zero-padded */
@@ -204,8 +204,8 @@ bit_out(PG_FUNCTION_ARGS)
 	}
 
 	/*
-	 * Go back one step if we printed a hex number that was not part of
-	 * the bitstring anymore
+	 * Go back one step if we printed a hex number that was not part of the
+	 * bitstring anymore
 	 */
 	if (i > len)
 		r--;
@@ -221,8 +221,50 @@ bit_out(PG_FUNCTION_ARGS)
 Datum
 bit_recv(PG_FUNCTION_ARGS)
 {
-	/* Exactly the same as varbit_recv, so share code */
-	return varbit_recv(fcinfo);
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+#ifdef NOT_USED
+	Oid			typelem = PG_GETARG_OID(1);
+#endif
+	int32		atttypmod = PG_GETARG_INT32(2);
+	VarBit	   *result;
+	int			len,
+				bitlen;
+	int			ipad;
+	bits8		mask;
+
+	bitlen = pq_getmsgint(buf, sizeof(int32));
+	if (bitlen < 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_BINARY_REPRESENTATION),
+				 errmsg("invalid length in external bit string")));
+
+	/*
+	 * Sometimes atttypmod is not supplied. If it is supplied we need to make
+	 * sure that the bitstring fits.
+	 */
+	if (atttypmod > 0 && bitlen != atttypmod)
+		ereport(ERROR,
+				(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
+				 errmsg("bit string length %d does not match type bit(%d)",
+						bitlen, atttypmod)));
+
+	len = VARBITTOTALLEN(bitlen);
+	result = (VarBit *) palloc(len);
+	VARATT_SIZEP(result) = len;
+	VARBITLEN(result) = bitlen;
+
+	pq_copymsgbytes(buf, (char *) VARBITS(result), VARBITBYTES(result));
+
+	/* Make sure last byte is zero-padded if needed */
+	ipad = VARBITPAD(result);
+	if (ipad > 0)
+	{
+		mask = BITMASK << ipad;
+		*(VARBITS(result) + VARBITBYTES(result) - 1) &= mask;
+	}
+
+	PG_RETURN_VARBIT_P(result);
 }
 
 /*
@@ -260,8 +302,8 @@ bit(PG_FUNCTION_ARGS)
 	if (!isExplicit)
 		ereport(ERROR,
 				(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
-			   errmsg("bit string length %d does not match type bit(%d)",
-					  VARBITLEN(arg), len)));
+				 errmsg("bit string length %d does not match type bit(%d)",
+						VARBITLEN(arg), len)));
 
 	rlen = VARBITTOTALLEN(len);
 	/* set to 0 so that string is zero-padded */
@@ -273,9 +315,9 @@ bit(PG_FUNCTION_ARGS)
 		   Min(VARBITBYTES(result), VARBITBYTES(arg)));
 
 	/*
-	 * Make sure last byte is zero-padded if needed.  This is useless but
-	 * safe if source data was shorter than target length (we assume the
-	 * last byte of the source data was itself correctly zero-padded).
+	 * Make sure last byte is zero-padded if needed.  This is useless but safe
+	 * if source data was shorter than target length (we assume the last byte
+	 * of the source data was itself correctly zero-padded).
 	 */
 	ipad = VARBITPAD(result);
 	if (ipad > 0)
@@ -337,8 +379,8 @@ varbit_in(PG_FUNCTION_ARGS)
 		bitlen = slen * 4;
 
 	/*
-	 * Sometimes atttypmod is not supplied. If it is supplied we need to
-	 * make sure that the bitstring fits.
+	 * Sometimes atttypmod is not supplied. If it is supplied we need to make
+	 * sure that the bitstring fits.
 	 */
 	if (atttypmod <= 0)
 		atttypmod = bitlen;
@@ -464,6 +506,11 @@ Datum
 varbit_recv(PG_FUNCTION_ARGS)
 {
 	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+#ifdef NOT_USED
+	Oid			typelem = PG_GETARG_OID(1);
+#endif
+	int32		atttypmod = PG_GETARG_INT32(2);
 	VarBit	   *result;
 	int			len,
 				bitlen;
@@ -476,6 +523,16 @@ varbit_recv(PG_FUNCTION_ARGS)
 				(errcode(ERRCODE_INVALID_BINARY_REPRESENTATION),
 				 errmsg("invalid length in external bit string")));
 
+	/*
+	 * Sometimes atttypmod is not supplied. If it is supplied we need to make
+	 * sure that the bitstring fits.
+	 */
+	if (atttypmod > 0 && bitlen > atttypmod)
+		ereport(ERROR,
+				(errcode(ERRCODE_STRING_DATA_RIGHT_TRUNCATION),
+				 errmsg("bit string too long for type bit varying(%d)",
+						atttypmod)));
+
 	len = VARBITTOTALLEN(bitlen);
 	result = (VarBit *) palloc(len);
 	VARATT_SIZEP(result) = len;
@@ -505,7 +562,7 @@ varbit_send(PG_FUNCTION_ARGS)
 
 	pq_begintypsend(&buf);
 	pq_sendint(&buf, VARBITLEN(s), sizeof(int32));
-	pq_sendbytes(&buf, VARBITS(s), VARBITBYTES(s));
+	pq_sendbytes(&buf, (char *) VARBITS(s), VARBITBYTES(s));
 	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
 }
 
@@ -824,8 +881,8 @@ bitsubstr(PG_FUNCTION_ARGS)
 	else
 	{
 		/*
-		 * OK, we've got a true substring starting at position s1-1 and
-		 * ending at position e1-1
+		 * OK, we've got a true substring starting at position s1-1 and ending
+		 * at position e1-1
 		 */
 		rbitlen = e1 - s1;
 		len = VARBITTOTALLEN(rbitlen);
