@@ -20,7 +20,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_files.c,v 1.21.2.1 2007/08/06 01:38:57 tgl Exp $
+ *		$PostgreSQL: pgsql/src/bin/pg_dump/pg_backup_files.c,v 1.27.2.1 2007/08/06 01:38:32 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -101,7 +101,7 @@ InitArchiveFmt_Files(ArchiveHandle *AH)
 	/*
 	 * Set up some special context used in compressing data.
 	 */
-	ctx = (lclContext *) malloc(sizeof(lclContext));
+	ctx = (lclContext *) calloc(1, sizeof(lclContext));
 	AH->formatData = (void *) ctx;
 	ctx->filePos = 0;
 
@@ -167,16 +167,16 @@ _ArchiveEntry(ArchiveHandle *AH, TocEntry *te)
 	lclTocEntry *ctx;
 	char		fn[K_STD_BUF_SIZE];
 
-	ctx = (lclTocEntry *) malloc(sizeof(lclTocEntry));
+	ctx = (lclTocEntry *) calloc(1, sizeof(lclTocEntry));
 	if (te->dataDumper)
 	{
 #ifdef HAVE_LIBZ
 		if (AH->compression == 0)
-			sprintf(fn, "%d.dat", te->id);
+			sprintf(fn, "%d.dat", te->dumpId);
 		else
-			sprintf(fn, "%d.dat.gz", te->id);
+			sprintf(fn, "%d.dat.gz", te->dumpId);
 #else
-		sprintf(fn, "%d.dat", te->id);
+		sprintf(fn, "%d.dat", te->dumpId);
 #endif
 		ctx->filename = strdup(fn);
 	}
@@ -206,7 +206,7 @@ _ReadExtraToc(ArchiveHandle *AH, TocEntry *te)
 
 	if (ctx == NULL)
 	{
-		ctx = (lclTocEntry *) malloc(sizeof(lclTocEntry));
+		ctx = (lclTocEntry *) calloc(1, sizeof(lclTocEntry));
 		te->formatData = (void *) ctx;
 	}
 
@@ -224,7 +224,8 @@ _PrintExtraToc(ArchiveHandle *AH, TocEntry *te)
 {
 	lclTocEntry *ctx = (lclTocEntry *) te->formatData;
 
-	ahprintf(AH, "-- File: %s\n", ctx->filename);
+	if (AH->public.verbose)
+		ahprintf(AH, "-- File: %s\n", ctx->filename);
 }
 
 static void
@@ -457,7 +458,6 @@ _CloseArchive(ArchiveHandle *AH)
  * It is called just prior to the dumper's DataDumper routine.
  *
  * Optional, but strongly recommended.
- *
  */
 static void
 _StartBlobs(ArchiveHandle *AH, TocEntry *te)
@@ -470,7 +470,7 @@ _StartBlobs(ArchiveHandle *AH, TocEntry *te)
 
 	if (ctx->blobToc == NULL)
 		die_horribly(AH, modulename,
-					 "could not open large object TOC for output: %s\n", strerror(errno));
+		"could not open large object TOC for output: %s\n", strerror(errno));
 }
 
 /*
@@ -516,7 +516,6 @@ _StartBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
  * Called by the archiver when the dumper calls EndBlob.
  *
  * Optional.
- *
  */
 static void
 _EndBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
@@ -531,7 +530,6 @@ _EndBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
  * Called by the archiver when finishing saving all BLOB DATA.
  *
  * Optional.
- *
  */
 static void
 _EndBlobs(ArchiveHandle *AH, TocEntry *te)
@@ -543,5 +541,4 @@ _EndBlobs(ArchiveHandle *AH, TocEntry *te)
 
 	if (fclose(ctx->blobToc) != 0)
 		die_horribly(AH, modulename, "could not close large object TOC file: %s\n", strerror(errno));
-
 }
