@@ -5,10 +5,10 @@
  *	  along with the relation's initial contents.
  *
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: pg_constraint.h,v 1.9.4.1 2010/06/30 18:11:43 heikki Exp $
+ * $PostgreSQL: pgsql/src/include/catalog/pg_constraint.h,v 1.18.2.1 2005/11/22 18:23:27 momjian Exp $
  *
  * NOTES
  *	  the genbki.sh script reads this file and generates .bki
@@ -19,11 +19,9 @@
 #ifndef PG_CONSTRAINT_H
 #define PG_CONSTRAINT_H
 
-#include "nodes/pg_list.h"
-
 /* ----------------
  *		postgres.h contains the system type definitions and the
- *		CATALOG(), BOOTSTRAP and DATA() sugar words so this file
+ *		CATALOG(), BKI_BOOTSTRAP and DATA() sugar words so this file
  *		can be read by both genbki.sh and the C compiler.
  * ----------------
  */
@@ -33,16 +31,18 @@
  *		typedef struct FormData_pg_constraint
  * ----------------
  */
-CATALOG(pg_constraint)
+#define ConstraintRelationId  2606
+
+CATALOG(pg_constraint,2606)
 {
 	/*
 	 * conname + connamespace is deliberately not unique; we allow, for
 	 * example, the same name to be used for constraints of different
 	 * relations.  This is partly for backwards compatibility with past
-	 * Postgres practice, and partly because we don't want to have to
-	 * obtain a global lock to generate a globally unique name for a
-	 * nameless constraint.  We associate a namespace with constraint
-	 * names only for SQL92 compatibility.
+	 * Postgres practice, and partly because we don't want to have to obtain a
+	 * global lock to generate a globally unique name for a nameless
+	 * constraint.	We associate a namespace with constraint names only for
+	 * SQL92 compatibility.
 	 */
 	NameData	conname;		/* name of this constraint */
 	Oid			connamespace;	/* OID of namespace containing constraint */
@@ -51,9 +51,9 @@ CATALOG(pg_constraint)
 	bool		condeferred;	/* deferred by default? */
 
 	/*
-	 * conrelid and conkey are only meaningful if the constraint applies
-	 * to a specific relation (this excludes domain constraints and
-	 * assertions).  Otherwise conrelid is 0 and conkey is NULL.
+	 * conrelid and conkey are only meaningful if the constraint applies to a
+	 * specific relation (this excludes domain constraints and assertions).
+	 * Otherwise conrelid is 0 and conkey is NULL.
 	 */
 	Oid			conrelid;		/* relation this constraint constrains */
 
@@ -62,14 +62,13 @@ CATALOG(pg_constraint)
 	 * constraint.	Otherwise it's 0.
 	 *
 	 * For SQL-style global ASSERTIONs, both conrelid and contypid would be
-	 * zero.  This is not presently supported, however.
+	 * zero. This is not presently supported, however.
 	 */
 	Oid			contypid;		/* domain this constraint constrains */
 
 	/*
 	 * These fields, plus confkey, are only meaningful for a foreign-key
-	 * constraint.	Otherwise confrelid is 0 and the char fields are
-	 * spaces.
+	 * constraint.	Otherwise confrelid is 0 and the char fields are spaces.
 	 */
 	Oid			confrelid;		/* relation referenced by foreign key */
 	char		confupdtype;	/* foreign key's ON UPDATE action */
@@ -143,15 +142,14 @@ typedef FormData_pg_constraint *Form_pg_constraint;
  */
 
 /*
- * Used for constraint support functions where the
- * and conrelid, contypid columns being looked up
+ * Identify constraint type for lookup purposes
  */
-typedef enum CONSTRAINTCATEGORY
+typedef enum ConstraintCategory
 {
 	CONSTRAINT_RELATION,
 	CONSTRAINT_DOMAIN,
-	CONSTRAINT_ASSERTION
-} CONSTRAINTCATEGORY;
+	CONSTRAINT_ASSERTION		/* for future expansion */
+} ConstraintCategory;
 
 /*
  * prototypes for functions in pg_constraint.c
@@ -178,10 +176,15 @@ extern Oid CreateConstraintEntry(const char *constraintName,
 
 extern void RemoveConstraintById(Oid conId);
 
-extern bool ConstraintNameIsUsed(CONSTRAINTCATEGORY conCat, Oid objId, Oid objNamespace,
-					 const char *cname);
-extern char *GenerateConstraintName(CONSTRAINTCATEGORY conCat, Oid objId, Oid objNamespace,
-					   int *counter);
-extern bool ConstraintNameIsGenerated(const char *cname);
+extern bool ConstraintNameIsUsed(ConstraintCategory conCat, Oid objId,
+					 Oid objNamespace, const char *conname);
+extern char *ChooseConstraintName(const char *name1, const char *name2,
+					 const char *label, Oid namespace,
+					 List *others);
+
+extern char *GetConstraintNameForTrigger(Oid triggerId);
+
+extern void AlterConstraintNamespaces(Oid ownerId, Oid oldNspId,
+						  Oid newNspId, bool isType);
 
 #endif   /* PG_CONSTRAINT_H */
