@@ -29,7 +29,7 @@
  * MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  *
  * IDENTIFICATION
- *	$PostgreSQL: pgsql/src/pl/plpython/plpython.c,v 1.58.4.11 2010/04/30 19:16:19 tgl Exp $
+ *	$PostgreSQL: pgsql/src/pl/plpython/plpython.c,v 1.66.2.7 2009/11/03 08:12:46 petere Exp $
  *
  *********************************************************************
  */
@@ -63,6 +63,7 @@ typedef int Py_ssize_t;
 #include "parser/parse_type.h"
 #include "tcop/tcopprot.h"
 #include "utils/lsyscache.h"
+#include "utils/memutils.h"
 #include "utils/syscache.h"
 #include "utils/typcache.h"
 
@@ -128,8 +129,8 @@ typedef struct PLyTypeInfo
 	int			is_rowtype;
 
 	/*
-	 * is_rowtype can be: -1  not known yet (initial state) 0  scalar
-	 * datatype 1  rowtype 2  rowtype, but I/O functions not set up yet
+	 * is_rowtype can be: -1  not known yet (initial state) 0  scalar datatype
+	 * 1  rowtype 2  rowtype, but I/O functions not set up yet
 	 */
 }	PLyTypeInfo;
 
@@ -143,8 +144,8 @@ typedef struct PLyProcedure
 	TransactionId fn_xmin;
 	CommandId	fn_cmin;
 	bool		fn_readonly;
-	PLyTypeInfo result;			/* also used to store info for trigger
-								 * tuple type */
+	PLyTypeInfo result;			/* also used to store info for trigger tuple
+								 * type */
 	PLyTypeInfo args[FUNC_MAX_ARGS];
 	int			nargs;
 	PyObject   *code;			/* compiled procedure code */
@@ -347,7 +348,7 @@ plpython_call_handler(PG_FUNCTION_ARGS)
 			HeapTuple	trv;
 
 			proc = PLy_procedure_get(fcinfo,
-								   RelationGetRelid(tdata->tg_relation));
+									 RelationGetRelid(tdata->tg_relation));
 			PLy_curr_procedure = proc;
 			trv = PLy_trigger_handler(fcinfo, proc);
 			retval = PointerGetDatum(trv);
@@ -437,8 +438,8 @@ PLy_trigger_handler(FunctionCallInfo fcinfo, PLyProcedure * proc)
 			{
 				/*
 				 * hmmm, perhaps they only read the pltcl page, not a
-				 * surprising thing since i've written no documentation,
-				 * so accept a belated OK
+				 * surprising thing since i've written no documentation, so
+				 * accept a belated OK
 				 */
 				elog(ERROR, "expected return to be \"SKIP\" or \"MODIFY\"");
 			}
@@ -530,8 +531,8 @@ PLy_modify_tuple(PLyProcedure * proc, PyObject * pltd, TriggerData *tdata,
 
 				modvalues[i] = FunctionCall3(&proc->result.out.r.atts[atti].typfunc,
 											 CStringGetDatum(src),
-											 ObjectIdGetDatum(proc->result.out.r.atts[atti].typioparam),
-						 Int32GetDatum(tupdesc->attrs[atti]->atttypmod));
+				  ObjectIdGetDatum(proc->result.out.r.atts[atti].typioparam),
+							 Int32GetDatum(tupdesc->attrs[atti]->atttypmod));
 				modnulls[i] = ' ';
 
 				Py_DECREF(plstr);
@@ -606,7 +607,7 @@ PLy_trigger_build_args(FunctionCallInfo fcinfo, PLyProcedure * proc, HeapTuple *
 		Py_DECREF(pltname);
 
 		stroid = DatumGetCString(DirectFunctionCall1(oidout,
-						   ObjectIdGetDatum(tdata->tg_relation->rd_id)));
+							   ObjectIdGetDatum(tdata->tg_relation->rd_id)));
 		pltrelid = PyString_FromString(stroid);
 		PyDict_SetItemString(pltdata, "relid", pltrelid);
 		Py_DECREF(pltrelid);
@@ -762,10 +763,10 @@ PLy_function_handler(FunctionCallInfo fcinfo, PLyProcedure * proc)
 		Assert(!PLy_error_in_progress);
 
 		/*
-		 * Disconnect from SPI manager and then create the return values
-		 * datum (if the input function does a palloc for it this must not
-		 * be allocated in the SPI memory context because SPI_finish would
-		 * free it).
+		 * Disconnect from SPI manager and then create the return values datum
+		 * (if the input function does a palloc for it this must not be
+		 * allocated in the SPI memory context because SPI_finish would free
+		 * it).
 		 */
 		if (SPI_finish() != SPI_OK_FINISH)
 			elog(ERROR, "SPI_finish failed");
@@ -787,7 +788,7 @@ PLy_function_handler(FunctionCallInfo fcinfo, PLyProcedure * proc)
 			plrv_sc = PyString_AsString(plrv_so);
 			rv = FunctionCall3(&proc->result.out.d.typfunc,
 							   PointerGetDatum(plrv_sc),
-						 ObjectIdGetDatum(proc->result.out.d.typioparam),
+							 ObjectIdGetDatum(proc->result.out.d.typioparam),
 							   Int32GetDatum(-1));
 		}
 
@@ -819,8 +820,8 @@ PLy_procedure_call(PLyProcedure * proc, char *kargs, PyObject * vargs)
 						 proc->globals, proc->globals);
 
 	/*
-	 * If there was an error in a PG callback, propagate that no matter
-	 * what Python claims about its success.
+	 * If there was an error in a PG callback, propagate that no matter what
+	 * Python claims about its success.
 	 */
 	if (PLy_error_in_progress)
 	{
@@ -893,7 +894,7 @@ PLy_function_build_args(FunctionCallInfo fcinfo, PLyProcedure * proc)
 
 					dt = FunctionCall3(&(proc->args[i].in.d.typfunc),
 									   fcinfo->arg[i],
-						 ObjectIdGetDatum(proc->args[i].in.d.typioparam),
+							 ObjectIdGetDatum(proc->args[i].in.d.typioparam),
 									   Int32GetDatum(-1));
 					ct = DatumGetCString(dt);
 					arg = (proc->args[i].in.d.func) (ct);
@@ -1034,8 +1035,8 @@ PLy_procedure_create(FunctionCallInfo fcinfo, Oid tgreloid,
 	PG_TRY();
 	{
 		/*
-		 * get information required for output conversion of the return
-		 * value, but only if this isn't a trigger.
+		 * get information required for output conversion of the return value,
+		 * but only if this isn't a trigger.
 		 */
 		if (!CALLED_AS_TRIGGER(fcinfo))
 		{
@@ -1043,7 +1044,7 @@ PLy_procedure_create(FunctionCallInfo fcinfo, Oid tgreloid,
 			Form_pg_type rvTypeStruct;
 
 			rvTypeTup = SearchSysCache(TYPEOID,
-								ObjectIdGetDatum(procStruct->prorettype),
+									ObjectIdGetDatum(procStruct->prorettype),
 									   0, 0, 0);
 			if (!HeapTupleIsValid(rvTypeTup))
 				elog(ERROR, "cache lookup failed for type %u",
@@ -1060,14 +1061,14 @@ PLy_procedure_create(FunctionCallInfo fcinfo, Oid tgreloid,
 				else
 					ereport(ERROR,
 							(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-							 errmsg("plpython functions cannot return type %s",
-							   format_type_be(procStruct->prorettype))));
+						   errmsg("plpython functions cannot return type %s",
+								  format_type_be(procStruct->prorettype))));
 			}
 
 			if (rvTypeStruct->typtype == 'c')
 				ereport(ERROR,
 						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-					errmsg("plpython functions cannot return tuples yet")));
+					 errmsg("plpython functions cannot return tuples yet")));
 			else
 				PLy_output_datum_func(&proc->result, rvTypeTup);
 
@@ -1086,8 +1087,8 @@ PLy_procedure_create(FunctionCallInfo fcinfo, Oid tgreloid,
 		}
 
 		/*
-		 * now get information required for input conversion of the
-		 * procedure's arguments.
+		 * now get information required for input conversion of the procedure's
+		 * arguments.
 		 */
 		proc->nargs = fcinfo->nargs;
 		for (i = 0; i < fcinfo->nargs; i++)
@@ -1096,11 +1097,11 @@ PLy_procedure_create(FunctionCallInfo fcinfo, Oid tgreloid,
 			Form_pg_type argTypeStruct;
 
 			argTypeTup = SearchSysCache(TYPEOID,
-							ObjectIdGetDatum(procStruct->proargtypes[i]),
+						 ObjectIdGetDatum(procStruct->proargtypes.values[i]),
 										0, 0, 0);
 			if (!HeapTupleIsValid(argTypeTup))
 				elog(ERROR, "cache lookup failed for type %u",
-					 procStruct->proargtypes[i]);
+					 procStruct->proargtypes.values[i]);
 			argTypeStruct = (Form_pg_type) GETSTRUCT(argTypeTup);
 
 			/* Disallow pseudotype argument */
@@ -1108,15 +1109,14 @@ PLy_procedure_create(FunctionCallInfo fcinfo, Oid tgreloid,
 				ereport(ERROR,
 						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 						 errmsg("plpython functions cannot take type %s",
-								format_type_be(procStruct->proargtypes[i]))));
+						format_type_be(procStruct->proargtypes.values[i]))));
 
 			if (argTypeStruct->typtype != 'c')
 				PLy_input_datum_func(&(proc->args[i]),
-									 procStruct->proargtypes[i],
+									 procStruct->proargtypes.values[i],
 									 argTypeTup);
 			else
-				proc->args[i].is_rowtype = 2;	/* still need to set I/O
-												 * funcs */
+				proc->args[i].is_rowtype = 2;	/* still need to set I/O funcs */
 
 			ReleaseSysCache(argTypeTup);
 		}
@@ -1161,8 +1161,8 @@ PLy_procedure_compile(PLyProcedure * proc, const char *src)
 	proc->globals = PyDict_Copy(PLy_interp_globals);
 
 	/*
-	 * SD is private preserved data between calls GD is global data shared
-	 * by all functions
+	 * SD is private preserved data between calls GD is global data shared by
+	 * all functions
 	 */
 	proc->statics = PyDict_New();
 	PyDict_SetItemString(proc->globals, "SD", proc->statics);
@@ -1288,7 +1288,7 @@ PLy_input_tuple_funcs(PLyTypeInfo * arg, TupleDesc desc)
 			continue;
 
 		typeTup = SearchSysCache(TYPEOID,
-							  ObjectIdGetDatum(desc->attrs[i]->atttypid),
+								 ObjectIdGetDatum(desc->attrs[i]->atttypid),
 								 0, 0, 0);
 		if (!HeapTupleIsValid(typeTup))
 			elog(ERROR, "cache lookup failed for type %u",
@@ -1322,7 +1322,7 @@ PLy_output_tuple_funcs(PLyTypeInfo * arg, TupleDesc desc)
 			continue;
 
 		typeTup = SearchSysCache(TYPEOID,
-							  ObjectIdGetDatum(desc->attrs[i]->atttypid),
+								 ObjectIdGetDatum(desc->attrs[i]->atttypid),
 								 0, 0, 0);
 		if (!HeapTupleIsValid(typeTup))
 			elog(ERROR, "cache lookup failed for type %u",
@@ -1501,8 +1501,8 @@ PLyDict_FromTuple(PLyTypeInfo * info, HeapTuple tuple, TupleDesc desc)
 			{
 				vdat = FunctionCall3(&info->in.r.atts[i].typfunc,
 									 vattr,
-						 ObjectIdGetDatum(info->in.r.atts[i].typioparam),
-							   Int32GetDatum(desc->attrs[i]->atttypmod));
+							 ObjectIdGetDatum(info->in.r.atts[i].typioparam),
+								   Int32GetDatum(desc->attrs[i]->atttypmod));
 				vsrc = DatumGetCString(vdat);
 
 				/*
@@ -1560,8 +1560,8 @@ static int	PLy_result_ass_slice(PyObject *, Py_ssize_t, Py_ssize_t, PyObject *);
 
 static PyObject *PLy_spi_prepare(PyObject *, PyObject *);
 static PyObject *PLy_spi_execute(PyObject *, PyObject *);
-static PyObject *PLy_spi_execute_query(char *query, int limit);
-static PyObject *PLy_spi_execute_plan(PyObject *, PyObject *, int);
+static PyObject *PLy_spi_execute_query(char *query, long limit);
+static PyObject *PLy_spi_execute_plan(PyObject *, PyObject *, long);
 static PyObject *PLy_spi_execute_fetch_result(SPITupleTable *, int, int);
 
 
@@ -1853,7 +1853,7 @@ PLy_spi_prepare(PyObject * self, PyObject * args)
 	PyObject   *volatile optr = NULL;
 	char	   *query;
 	void	   *tmpplan;
-	volatile MemoryContext oldcontext;
+	MemoryContext oldcontext;
 
 	/* Can't execute more if we have an unhandled error */
 	if (PLy_error_in_progress)
@@ -1872,7 +1872,7 @@ PLy_spi_prepare(PyObject * self, PyObject * args)
 	if ((list) && (!PySequence_Check(list)))
 	{
 		PyErr_SetString(PLy_exc_spi_error,
-				 "Second argument in plpy.prepare() must be a sequence");
+					 "Second argument in plpy.prepare() must be a sequence");
 		return NULL;
 	}
 
@@ -1897,8 +1897,8 @@ PLy_spi_prepare(PyObject * self, PyObject * args)
 
 				/*
 				 * the other loop might throw an exception, if PLyTypeInfo
-				 * member isn't properly initialized the Py_DECREF(plan)
-				 * will go boom
+				 * member isn't properly initialized the Py_DECREF(plan) will
+				 * go boom
 				 */
 				for (i = 0; i < nargs; i++)
 				{
@@ -1918,8 +1918,7 @@ PLy_spi_prepare(PyObject * self, PyObject * args)
 					sptr = PyString_AsString(optr);
 
 					/*
-					 * XXX should extend this to allow qualified type
-					 * names
+					 * XXX should extend this to allow qualified type names
 					 */
 					typeTup = typenameType(makeTypeName(sptr));
 					Py_DECREF(optr);
@@ -1978,7 +1977,7 @@ PLy_spi_execute(PyObject * self, PyObject * args)
 	char	   *query;
 	PyObject   *plan;
 	PyObject   *list = NULL;
-	int			limit = 0;
+	long		limit = 0;
 
 	/* Can't execute more if we have an unhandled error */
 	if (PLy_error_in_progress)
@@ -1987,12 +1986,12 @@ PLy_spi_execute(PyObject * self, PyObject * args)
 		return NULL;
 	}
 
-	if (PyArg_ParseTuple(args, "s|i", &query, &limit))
+	if (PyArg_ParseTuple(args, "s|l", &query, &limit))
 		return PLy_spi_execute_query(query, limit);
 
 	PyErr_Clear();
 
-	if ((PyArg_ParseTuple(args, "O|Oi", &plan, &list, &limit)) &&
+	if ((PyArg_ParseTuple(args, "O|Ol", &plan, &list, &limit)) &&
 		(is_PLyPlanObject(plan)))
 		return PLy_spi_execute_plan(plan, list, limit);
 
@@ -2001,13 +2000,13 @@ PLy_spi_execute(PyObject * self, PyObject * args)
 }
 
 static PyObject *
-PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
+PLy_spi_execute_plan(PyObject * ob, PyObject * list, long limit)
 {
 	volatile int nargs;
 	int			i,
 				rv;
 	PLyPlanObject *plan;
-	volatile MemoryContext oldcontext;
+	MemoryContext oldcontext;
 
 	if (list != NULL)
 	{
@@ -2069,7 +2068,7 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 					plan->values[j] =
 						FunctionCall3(&(plan->args[j].out.d.typfunc),
 									  CStringGetDatum(sv),
-							ObjectIdGetDatum(plan->args[j].out.d.typioparam),
+								ObjectIdGetDatum(plan->args[j].out.d.typioparam),
 									  Int32GetDatum(-1));
 				}
 				PG_CATCH();
@@ -2148,10 +2147,10 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 }
 
 static PyObject *
-PLy_spi_execute_query(char *query, int limit)
+PLy_spi_execute_query(char *query, long limit)
 {
 	int			rv;
-	volatile MemoryContext oldcontext;
+	MemoryContext oldcontext;
 
 	oldcontext = CurrentMemoryContext;
 	PG_TRY();
@@ -2188,7 +2187,7 @@ static PyObject *
 PLy_spi_execute_fetch_result(SPITupleTable *tuptable, int rows, int status)
 {
 	PLyResultObject *result;
-	volatile MemoryContext oldcontext;
+	MemoryContext oldcontext;
 
 	result = (PLyResultObject *) PLy_result_new();
 	Py_DECREF(result->status);
@@ -2209,9 +2208,9 @@ PLy_spi_execute_fetch_result(SPITupleTable *tuptable, int rows, int status)
 		PLyTypeInfo args;
 		int			i;
 
+		PLy_typeinfo_init(&args);
 		Py_DECREF(result->nrows);
 		result->nrows = PyInt_FromLong(rows);
-		PLy_typeinfo_init(&args);
 
 		oldcontext = CurrentMemoryContext;
 		PG_TRY();
@@ -2225,10 +2224,13 @@ PLy_spi_execute_fetch_result(SPITupleTable *tuptable, int rows, int status)
 				for (i = 0; i < rows; i++)
 				{
 					PyObject   *row = PLyDict_FromTuple(&args, tuptable->vals[i],
-													  tuptable->tupdesc);
+														tuptable->tupdesc);
 
 					PyList_SetItem(result->rows, i, row);
 				}
+				PLy_typeinfo_dealloc(&args);
+
+				SPI_freetuptable(tuptable);
 			}
 		}
 		PG_CATCH();
@@ -2238,16 +2240,12 @@ PLy_spi_execute_fetch_result(SPITupleTable *tuptable, int rows, int status)
 			FlushErrorState();
 			if (!PyErr_Occurred())
 				PyErr_SetString(PLy_exc_error,
-						"Unknown error in PLy_spi_execute_fetch_result");
-			PLy_typeinfo_dealloc(&args);
-			SPI_freetuptable(tuptable);
+							"Unknown error in PLy_spi_execute_fetch_result");
 			Py_DECREF(result);
+			PLy_typeinfo_dealloc(&args);
 			return NULL;
 		}
 		PG_END_TRY();
-
-		PLy_typeinfo_dealloc(&args);
-		SPI_freetuptable(tuptable);
 	}
 
 	return (PyObject *) result;
@@ -2451,8 +2449,7 @@ PLy_output(volatile int level, PyObject * self, PyObject * args)
 	Py_XDECREF(so);
 
 	/*
-	 * return a legal object so the interpreter will continue on its merry
-	 * way
+	 * return a legal object so the interpreter will continue on its merry way
 	 */
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -2574,10 +2571,10 @@ PLy_traceback(int *xlevel)
 		vstr = "Unknown";
 
 	/*
-	 * I'm not sure what to do if eob is NULL here -- we can't call
-	 * PLy_elog because that function calls us, so we could end up
-	 * with infinite recursion.  I'm not even sure if eob could be
-	 * NULL here -- would an Assert() be more appropriate?
+	 * I'm not sure what to do if eob is NULL here -- we can't call PLy_elog
+	 * because that function calls us, so we could end up with infinite
+	 * recursion.  I'm not even sure if eob could be NULL here -- would an
+	 * Assert() be more appropriate?
 	 */
 	estr = eob ? PyString_AsString(eob) : "Unknown Exception";
 	initStringInfo(&xstr);
@@ -2597,7 +2594,7 @@ PLy_traceback(int *xlevel)
 	else
 		*xlevel = ERROR;
 
-	Py_XDECREF(e);
+	Py_DECREF(e);
 	return xstr.data;
 }
 
