@@ -1,20 +1,25 @@
 --
+-- SELECT
+--
+
 -- btree index
 -- awk '{if($1<10){print;}else{next;}}' onek.data | sort +0n -1
 --
-SELECT onek.* WHERE onek.unique1 < 10;
+SELECT * FROM onek
+   WHERE onek.unique1 < 10
+   ORDER BY onek.unique1;
 
 --
 -- awk '{if($1<20){print $1,$14;}else{next;}}' onek.data | sort +0nr -1
 --
-SELECT onek.unique1, onek.stringu1
+SELECT onek.unique1, onek.stringu1 FROM onek
    WHERE onek.unique1 < 20 
    ORDER BY unique1 using >;
 
 --
 -- awk '{if($1>980){print $1,$14;}else{next;}}' onek.data | sort +1d -2
 --
-SELECT onek.unique1, onek.stringu1
+SELECT onek.unique1, onek.stringu1 FROM onek
    WHERE onek.unique1 > 980 
    ORDER BY stringu1 using <;
 	
@@ -22,7 +27,7 @@ SELECT onek.unique1, onek.stringu1
 -- awk '{if($1>980){print $1,$16;}else{next;}}' onek.data |
 -- sort +1d -2 +0nr -1
 --
-SELECT onek.unique1, onek.string4
+SELECT onek.unique1, onek.string4 FROM onek
    WHERE onek.unique1 > 980 
    ORDER BY string4 using <, unique1 using >;
 	
@@ -30,7 +35,7 @@ SELECT onek.unique1, onek.string4
 -- awk '{if($1>980){print $1,$16;}else{next;}}' onek.data |
 -- sort +1dr -2 +0n -1
 --
-SELECT onek.unique1, onek.string4
+SELECT onek.unique1, onek.string4 FROM onek
    WHERE onek.unique1 > 980
    ORDER BY string4 using >, unique1 using <;
 	
@@ -38,7 +43,7 @@ SELECT onek.unique1, onek.string4
 -- awk '{if($1<20){print $1,$16;}else{next;}}' onek.data |
 -- sort +0nr -1 +1d -2
 --
-SELECT onek.unique1, onek.string4
+SELECT onek.unique1, onek.string4 FROM onek
    WHERE onek.unique1 < 20
    ORDER BY unique1 using >, string4 using <;
 
@@ -46,33 +51,39 @@ SELECT onek.unique1, onek.string4
 -- awk '{if($1<20){print $1,$16;}else{next;}}' onek.data |
 -- sort +0n -1 +1dr -2
 --
-SELECT onek.unique1, onek.string4
+SELECT onek.unique1, onek.string4 FROM onek
    WHERE onek.unique1 < 20 
    ORDER BY unique1 using <, string4 using >;
 
 --
--- partial btree index
--- awk '{if($1<10){print $0;}else{next;}}' onek.data | sort +0n -1
+-- test partial btree indexes
 --
---SELECT onek2.* WHERE onek2.unique1 < 10;
+-- As of 7.2, planner probably won't pick an indexscan without stats,
+-- so ANALYZE first.
+--
+ANALYZE onek2;
+
+--
+-- awk '{if($1<10){print $0;}else{next;}}' onek.data | sort +0n -1
+--
+SELECT onek2.* FROM onek2 WHERE onek2.unique1 < 10;
 
 --
--- partial btree index
 -- awk '{if($1<20){print $1,$14;}else{next;}}' onek.data | sort +0nr -1
 --
---SELECT onek2.unique1, onek2.stringu1
---    WHERE onek2.unique1 < 20 
---    ORDER BY unique1 using >;
+SELECT onek2.unique1, onek2.stringu1 FROM onek2
+    WHERE onek2.unique1 < 20 
+    ORDER BY unique1 using >;
 
 --
 -- awk '{if($1>980){print $1,$14;}else{next;}}' onek.data | sort +1d -2
 --
---SELECT onek2.unique1, onek2.stringu1
---   WHERE onek2.unique1 > 980
---   ORDER BY stringu1 using <;
-	
+SELECT onek2.unique1, onek2.stringu1 FROM onek2
+   WHERE onek2.unique1 > 980;
+
+
 SELECT two, stringu1, ten, string4
-   INTO TABLE temp
+   INTO TABLE tmp
    FROM onek;
 
 --
@@ -93,3 +104,9 @@ SELECT p.name, p.age FROM person* p;
 --
 SELECT p.name, p.age FROM person* p ORDER BY age using >, name;
 
+--
+-- Test some cases involving whole-row Var referencing a subquery
+--
+select foo from (select 1) as foo;
+select foo from (select null) as foo;
+select foo from (select 'xyzzy',1,null) as foo;
