@@ -6,7 +6,7 @@
  * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/timezone/pgtz.c,v 1.29.4.1 2008/07/01 03:41:25 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/timezone/pgtz.c,v 1.38.2.2 2008/07/01 03:41:18 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -26,12 +26,18 @@
 #include "utils/datetime.h"
 #include "utils/elog.h"
 #include "utils/guc.h"
+#include "utils/hsearch.h"
+
+/* Current global timezone */
+pg_tz	   *global_timezone = NULL;
 
 
 static char tzdir[MAXPGPATH];
 static int	done_tzdir = 0;
 
 static const char *identify_system_timezone(void);
+static const char *select_default_timezone(void);
+static bool set_global_timezone(const char *tzname);
 
 
 /*
@@ -156,12 +162,23 @@ score_timezone(const char *tzname, struct tztry * tt)
 	struct tm  *systm;
 	struct pg_tm *pgtm;
 	char		cbuf[TZ_STRLEN_MAX + 1];
+	pg_tz		tz;
 
-	if (!pg_tzset(tzname))
-		return -1;				/* can't handle the TZ name at all */
+
+	/*
+	 * Load timezone directly. Don't use pg_tzset, because we don't want all
+	 * timezones loaded in the cache at startup.
+	 */
+	if (tzload(tzname, &tz.state) != 0)
+	{
+		if (tzname[0] == ':' || tzparse(tzname, &tz.state, FALSE) != 0)
+		{
+			return -1;			/* can't handle the TZ name at all */
+		}
+	}
 
 	/* Reject if leap seconds involved */
-	if (!tz_acceptable())
+	if (!tz_acceptable(&tz))
 	{
 		elog(DEBUG4, "Reject TZ \"%s\": uses leap seconds", tzname);
 		return -1;
@@ -171,7 +188,7 @@ score_timezone(const char *tzname, struct tztry * tt)
 	for (i = 0; i < tt->n_test_times; i++)
 	{
 		pgtt = (pg_time_t) (tt->test_times[i]);
-		pgtm = pg_localtime(&pgtt);
+		pgtm = pg_localtime(&pgtt, &tz);
 		if (!pgtm)
 			return -1;			/* probably shouldn't happen */
 		systm = localtime(&(tt->test_times[i]));
@@ -307,12 +324,12 @@ identify_system_timezone(void)
 	}
 
 	/*
-	 * Couldn't find a match in the database, so next we try constructed
-	 * zone names (like "PST8PDT").
+	 * Couldn't find a match in the database, so next we try constructed zone
+	 * names (like "PST8PDT").
 	 *
-	 * First we need to determine the names of the local standard and
-	 * daylight zones.	The idea here is to scan forward from today until
-	 * we have seen both zones, if both are in use.
+	 * First we need to determine the names of the local standard and daylight
+	 * zones.  The idea here is to scan forward from today until we have seen
+	 * both zones, if both are in use.
 	 */
 	memset(std_zone_name, 0, sizeof(std_zone_name));
 	memset(dst_zone_name, 0, sizeof(dst_zone_name));
@@ -321,15 +338,15 @@ identify_system_timezone(void)
 	tnow = time(NULL);
 
 	/*
-	 * Round back to a GMT midnight so results don't depend on local time
-	 * of day
+	 * Round back to a GMT midnight so results don't depend on local time of
+	 * day
 	 */
 	tnow -= (tnow % T_DAY);
 
 	/*
-	 * We have to look a little further ahead than one year, in case today
-	 * is just past a DST boundary that falls earlier in the year than the
-	 * next similar boundary.  Arbitrarily scan up to 14 months.
+	 * We have to look a little further ahead than one year, in case today is
+	 * just past a DST boundary that falls earlier in the year than the next
+	 * similar boundary.  Arbitrarily scan up to 14 months.
 	 */
 	for (t = tnow; t <= tnow + T_MONTH * 14; t += T_MONTH)
 	{
@@ -363,7 +380,7 @@ identify_system_timezone(void)
 	{
 		ereport(LOG,
 				(errmsg("unable to determine system timezone, defaulting to \"%s\"", "GMT"),
-				 errhint("You can specify the correct timezone in postgresql.conf.")));
+		errhint("You can specify the correct timezone in postgresql.conf.")));
 		return NULL;			/* go to GMT */
 	}
 
@@ -388,19 +405,18 @@ identify_system_timezone(void)
 		return resultbuf;
 
 	/*
-	 * Did not find the timezone.  Fallback to use a GMT zone.	Note that
-	 * the zic timezone database names the GMT-offset zones in POSIX
-	 * style: plus is west of Greenwich.  It's unfortunate that this is
-	 * opposite of SQL conventions.  Should we therefore change the names?
-	 * Probably not...
+	 * Did not find the timezone.  Fallback to use a GMT zone.	Note that the
+	 * zic timezone database names the GMT-offset zones in POSIX style: plus
+	 * is west of Greenwich.  It's unfortunate that this is opposite of SQL
+	 * conventions.  Should we therefore change the names? Probably not...
 	 */
 	snprintf(resultbuf, sizeof(resultbuf), "Etc/GMT%s%d",
 			 (-std_ofs > 0) ? "+" : "", -std_ofs / 3600);
 
 	ereport(LOG,
-	 (errmsg("could not recognize system timezone, defaulting to \"%s\"",
-			 resultbuf),
-	errhint("You can specify the correct timezone in postgresql.conf.")));
+		 (errmsg("could not recognize system timezone, defaulting to \"%s\"",
+				 resultbuf),
+	   errhint("You can specify the correct timezone in postgresql.conf.")));
 	return resultbuf;
 }
 
@@ -429,6 +445,7 @@ scan_available_timezones(char *tzdir, char *tzdirsub, struct tztry * tt,
 {
 	int			tzdir_orig_len = strlen(tzdir);
 	DIR		   *dirdesc;
+	struct dirent *direntry;
 
 	dirdesc = AllocateDir(tzdir);
 	if (!dirdesc)
@@ -439,22 +456,10 @@ scan_available_timezones(char *tzdir, char *tzdirsub, struct tztry * tt,
 		return;
 	}
 
-	for (;;)
+	while ((direntry = ReadDir(dirdesc, tzdir)) != NULL)
 	{
-		struct dirent *direntry;
 		struct stat statbuf;
 
-		errno = 0;
-		direntry = readdir(dirdesc);
-		if (!direntry)
-		{
-			if (errno)
-				ereport(LOG,
-						(errcode_for_file_access(),
-						 errmsg("error reading directory: %m")));
-			break;
-		}
-
 		/* Ignore . and .., plus any other "hidden" files */
 		if (direntry->d_name[0] == '.')
 			continue;
@@ -467,6 +472,7 @@ scan_available_timezones(char *tzdir, char *tzdirsub, struct tztry * tt,
 			ereport(LOG,
 					(errcode_for_file_access(),
 					 errmsg("could not stat \"%s\": %m", tzdir)));
+			tzdir[tzdir_orig_len] = '\0';
 			continue;
 		}
 
@@ -495,14 +501,13 @@ scan_available_timezones(char *tzdir, char *tzdirsub, struct tztry * tt,
 					StrNCpy(bestzonename, tzdirsub, TZ_STRLEN_MAX + 1);
 			}
 		}
+
+		/* Restore tzdir */
+		tzdir[tzdir_orig_len] = '\0';
 	}
 
 	FreeDir(dirdesc);
-
-	/* Restore tzdir */
-	tzdir[tzdir_orig_len] = '\0';
 }
-
 #else							/* WIN32 */
 
 static const struct
@@ -515,12 +520,11 @@ static const struct
 {
 	/*
 	 * This list was built from the contents of the registry at
-	 * HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows
-	 * NT\CurrentVersion\Time Zones on Windows XP Professional SP1
+	 * HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Time
+	 * Zones on Windows XP Professional SP1
 	 *
 	 * The zones have been matched to zic timezones by looking at the cities
-	 * listed in the win32 display name (in the comment here) in most
-	 * cases.
+	 * listed in the win32 display name (in the comment here) in most cases.
 	 */
 	{
 		"Afghanistan Standard Time", "Afghanistan Daylight Time",
@@ -585,8 +589,8 @@ static const struct
 	{
 		"Central Europe Standard Time", "Central Europe Daylight Time",
 		"Europe/Belgrade"
-	},							/* (GMT+01:00) Belgrade, Bratislava,
-								 * Budapest, Ljubljana, Prague */
+	},							/* (GMT+01:00) Belgrade, Bratislava, Budapest,
+								 * Ljubljana, Prague */
 	{
 		"Central European Standard Time", "Central European Daylight Time",
 		"Europe/Sarajevo"
@@ -604,13 +608,12 @@ static const struct
 	{
 		"China Standard Time", "China Daylight Time",
 		"Asia/Hong_Kong"
-	},							/* (GMT+08:00) Beijing, Chongqing, Hong
-								 * Kong, Urumqi */
+	},							/* (GMT+08:00) Beijing, Chongqing, Hong Kong,
+								 * Urumqi */
 	{
 		"Dateline Standard Time", "Dateline Daylight Time",
 		"Etc/GMT+12"
-	},							/* (GMT-12:00) International Date Line
-								 * West */
+	},							/* (GMT-12:00) International Date Line West */
 	{
 		"E. Africa Standard Time", "E. Africa Daylight Time",
 		"Africa/Nairobi"
@@ -642,13 +645,12 @@ static const struct
 	{
 		"Fiji Standard Time", "Fiji Daylight Time",
 		"Pacific/Fiji"
-	},							/* (GMT+12:00) Fiji, Kamchatka, Marshall
-								 * Is. */
+	},							/* (GMT+12:00) Fiji, Kamchatka, Marshall Is. */
 	{
 		"FLE Standard Time", "FLE Daylight Time",
 		"Europe/Helsinki"
-	},							/* (GMT+02:00) Helsinki, Kyiv, Riga,
-								 * Sofia, Tallinn, Vilnius */
+	},							/* (GMT+02:00) Helsinki, Kyiv, Riga, Sofia,
+								 * Tallinn, Vilnius */
 	{
 		"GMT Standard Time", "GMT Daylight Time",
 		"Europe/Dublin"
@@ -673,8 +675,8 @@ static const struct
 	{
 		"India Standard Time", "India Daylight Time",
 		"Asia/Calcutta"
-	},							/* (GMT+05:30) Chennai, Kolkata, Mumbai,
-								 * New Delhi */
+	},							/* (GMT+05:30) Chennai, Kolkata, Mumbai, New
+								 * Delhi */
 	{
 		"Iran Standard Time", "Iran Daylight Time",
 		"Asia/Tehran"
@@ -744,8 +746,8 @@ static const struct
 	{
 		"Romance Standard Time", "Romance Daylight Time",
 		"Europe/Brussels"
-	},							/* (GMT+01:00) Brussels, Copenhagen,
-								 * Madrid, Paris */
+	},							/* (GMT+01:00) Brussels, Copenhagen, Madrid,
+								 * Paris */
 	{
 		"Russian Standard Time", "Russian Daylight Time",
 		"Europe/Moscow"
@@ -816,18 +818,17 @@ static const struct
 		"Australia/Perth"
 	},							/* (GMT+08:00) Perth */
 /*	{"W. Central Africa Standard Time", "W. Central Africa Daylight Time",
-	 *	 *	""}, Could not find a match for this one. Excluded for now. *//* (
+	 *	 *	 *	 *	""}, Could not find a match for this one. Excluded for now. *//* (
 	 * G MT+01:00) West Central Africa */
 	{
 		"W. Europe Standard Time", "W. Europe Daylight Time",
 		"CET"
-	},							/* (GMT+01:00) Amsterdam, Berlin, Bern,
-								 * Rome, Stockholm, Vienna */
+	},							/* (GMT+01:00) Amsterdam, Berlin, Bern, Rome,
+								 * Stockholm, Vienna */
 	{
 		"West Asia Standard Time", "West Asia Daylight Time",
 		"Asia/Karachi"
-	},							/* (GMT+05:00) Islamabad, Karachi,
-								 * Tashkent */
+	},							/* (GMT+05:00) Islamabad, Karachi, Tashkent */
 	{
 		"West Pacific Standard Time", "West Pacific Daylight Time",
 		"Pacific/Guam"
@@ -846,11 +847,11 @@ identify_system_timezone(void)
 {
 	int			i;
 	char		tzname[128];
-	char        localtzname[256];
+	char		localtzname[256];
 	time_t		t = time(NULL);
 	struct tm  *tm = localtime(&t);
-	HKEY        rootKey;
-	int         idx;
+	HKEY		rootKey;
+	int			idx;
 
 	if (!tm)
 	{
@@ -874,62 +875,62 @@ identify_system_timezone(void)
 	}
 
 	/*
-	 * Localized Windows versions return localized names for the
-	 * timezone. Scan the registry to find the English name,
-	 * and then try matching against our table again.
+	 * Localized Windows versions return localized names for the timezone.
+	 * Scan the registry to find the English name, and then try matching
+	 * against our table again.
 	 */
 	memset(localtzname, 0, sizeof(localtzname));
 	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-					 "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
+			   "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
 					 0,
 					 KEY_READ,
 					 &rootKey) != ERROR_SUCCESS)
 	{
 		ereport(WARNING,
-				(errmsg_internal("could not open registry key to identify Windows timezone: %i", (int)GetLastError())));
+				(errmsg_internal("could not open registry key to identify Windows timezone: %i", (int) GetLastError())));
 		return NULL;
 	}
-	
-	for (idx = 0; ; idx++) 
+
+	for (idx = 0;; idx++)
 	{
-		char keyname[256];
-		char zonename[256];
-		DWORD namesize;
-		FILETIME lastwrite;
-		HKEY key;
-		LONG r;
-		
+		char		keyname[256];
+		char		zonename[256];
+		DWORD		namesize;
+		FILETIME	lastwrite;
+		HKEY		key;
+		LONG		r;
+
 		memset(keyname, 0, sizeof(keyname));
 		namesize = sizeof(keyname);
-		if ((r=RegEnumKeyEx(rootKey,
-							idx,
-							keyname,
-							&namesize,
-							NULL,
-							NULL,
-							NULL,
-							&lastwrite)) != ERROR_SUCCESS)
+		if ((r = RegEnumKeyEx(rootKey,
+							  idx,
+							  keyname,
+							  &namesize,
+							  NULL,
+							  NULL,
+							  NULL,
+							  &lastwrite)) != ERROR_SUCCESS)
 		{
 			if (r == ERROR_NO_MORE_ITEMS)
 				break;
 			ereport(WARNING,
-					(errmsg_internal("could not enumerate registry subkeys to identify Windows timezone: %i", (int)r)));
+					(errmsg_internal("could not enumerate registry subkeys to identify Windows timezone: %i", (int) r)));
 			break;
 		}
 
-		if ((r=RegOpenKeyEx(rootKey,keyname,0,KEY_READ,&key)) != ERROR_SUCCESS)
+		if ((r = RegOpenKeyEx(rootKey, keyname, 0, KEY_READ, &key)) != ERROR_SUCCESS)
 		{
 			ereport(WARNING,
-					(errmsg_internal("could not open registry subkey to identify Windows timezone: %i", (int)r)));
+					(errmsg_internal("could not open registry subkey to identify Windows timezone: %i", (int) r)));
 			break;
 		}
-		
+
 		memset(zonename, 0, sizeof(zonename));
 		namesize = sizeof(zonename);
-		if ((r=RegQueryValueEx(key, "Std", NULL, NULL, zonename, &namesize)) != ERROR_SUCCESS)
+		if ((r = RegQueryValueEx(key, "Std", NULL, NULL, zonename, &namesize)) != ERROR_SUCCESS)
 		{
 			ereport(WARNING,
-					(errmsg_internal("could not query value for 'std' to identify Windows timezone: %i", (int)r)));
+					(errmsg_internal("could not query value for 'std' to identify Windows timezone: %i", (int) r)));
 			RegCloseKey(key);
 			break;
 		}
@@ -942,10 +943,10 @@ identify_system_timezone(void)
 		}
 		memset(zonename, 0, sizeof(zonename));
 		namesize = sizeof(zonename);
-		if ((r=RegQueryValueEx(key, "Dlt", NULL, NULL, zonename, &namesize)) != ERROR_SUCCESS)
+		if ((r = RegQueryValueEx(key, "Dlt", NULL, NULL, zonename, &namesize)) != ERROR_SUCCESS)
 		{
 			ereport(WARNING,
-					(errmsg_internal("could not query value for 'dlt' to identify Windows timezone: %i", (int)r)));
+					(errmsg_internal("could not query value for 'dlt' to identify Windows timezone: %i", (int) r)));
 			RegCloseKey(key);
 			break;
 		}
@@ -985,6 +986,84 @@ identify_system_timezone(void)
 #endif   /* WIN32 */
 
 
+
+/*
+ * We keep loaded timezones in a hashtable so we don't have to
+ * load and parse the TZ definition file every time it is selected.
+ */
+static HTAB *timezone_cache = NULL;
+
+static bool
+init_timezone_hashtable(void)
+{
+	HASHCTL		hash_ctl;
+
+	MemSet(&hash_ctl, 0, sizeof(hash_ctl));
+
+	hash_ctl.keysize = TZ_STRLEN_MAX + 1;
+	hash_ctl.entrysize = sizeof(pg_tz);
+
+	timezone_cache = hash_create("Timezones",
+								 4,
+								 &hash_ctl,
+								 HASH_ELEM);
+	if (!timezone_cache)
+		return false;
+
+	return true;
+}
+
+/*
+ * Load a timezone from file or from cache.
+ * Does not verify that the timezone is acceptable!
+ */
+struct pg_tz *
+pg_tzset(const char *name)
+{
+	pg_tz	   *tzp;
+	pg_tz		tz;
+
+	if (strlen(name) > TZ_STRLEN_MAX)
+		return NULL;			/* not going to fit */
+
+	if (!timezone_cache)
+		if (!init_timezone_hashtable())
+			return NULL;
+
+	tzp = (pg_tz *) hash_search(timezone_cache,
+								name,
+								HASH_FIND,
+								NULL);
+	if (tzp)
+	{
+		/* Timezone found in cache, nothing more to do */
+		return tzp;
+	}
+
+	if (tzload(name, &tz.state) != 0)
+	{
+		if (name[0] == ':' || tzparse(name, &tz.state, FALSE) != 0)
+		{
+			/* Unknown timezone. Fail our call instead of loading GMT! */
+			return NULL;
+		}
+	}
+
+	strcpy(tz.TZname, name);
+
+	/* Save timezone in the cache */
+	tzp = hash_search(timezone_cache,
+					  name,
+					  HASH_ENTER,
+					  NULL);
+
+	strcpy(tzp->TZname, tz.TZname);
+	memcpy(&tzp->state, &tz.state, sizeof(tz.state));
+
+	return tzp;
+}
+
+
 /*
  * Check whether timezone is acceptable.
  *
@@ -996,24 +1075,47 @@ identify_system_timezone(void)
  * it can restore the old value of TZ if we don't like the new one.
  */
 bool
-tz_acceptable(void)
+tz_acceptable(pg_tz *tz)
 {
 	struct pg_tm *tt;
 	pg_time_t	time2000;
 
 	/*
-	 * To detect leap-second timekeeping, run pg_localtime for what should
-	 * be GMT midnight, 2000-01-01.  Insist that the tm_sec value be zero;
-	 * any other result has to be due to leap seconds.
+	 * To detect leap-second timekeeping, run pg_localtime for what should be
+	 * GMT midnight, 2000-01-01.  Insist that the tm_sec value be zero; any
+	 * other result has to be due to leap seconds.
 	 */
-	time2000 = (POSTGRES_EPOCH_JDATE - UNIX_EPOCH_JDATE) * 86400;
-	tt = pg_localtime(&time2000);
+	time2000 = (POSTGRES_EPOCH_JDATE - UNIX_EPOCH_JDATE) * SECS_PER_DAY;
+	tt = pg_localtime(&time2000, tz);
 	if (!tt || tt->tm_sec != 0)
 		return false;
 
 	return true;
 }
 
+
+/*
+ * Set the global timezone. Verify that it's acceptable first.
+ */
+static bool
+set_global_timezone(const char *tzname)
+{
+	pg_tz	   *tznew;
+
+	if (!tzname || !tzname[0])
+		return false;
+
+	tznew = pg_tzset(tzname);
+	if (!tznew)
+		return false;
+
+	if (!tz_acceptable(tznew))
+		return false;
+
+	global_timezone = tznew;
+	return true;
+}
+
 /*
  * Identify a suitable default timezone setting based on the environment,
  * and make it active.
@@ -1023,20 +1125,20 @@ tz_acceptable(void)
  * from the behavior of the system timezone library.  When all else fails,
  * fall back to GMT.
  */
-const char *
+static const char *
 select_default_timezone(void)
 {
 	const char *def_tz;
 
 	def_tz = getenv("TZ");
-	if (def_tz && pg_tzset(def_tz) && tz_acceptable())
+	if (set_global_timezone(def_tz))
 		return def_tz;
 
 	def_tz = identify_system_timezone();
-	if (def_tz && pg_tzset(def_tz) && tz_acceptable())
+	if (set_global_timezone(def_tz))
 		return def_tz;
 
-	if (pg_tzset("GMT") && tz_acceptable())
+	if (set_global_timezone("GMT"))
 		return "GMT";
 
 	ereport(FATAL,
